==> GotoBLAS-1.10.tar.gz: GotoBLAS/level3/hemm/zhemm.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

#ifdef XDOUBLE
#define ERROR_NAME "XHEMM "
#elif defined(DOUBLE)
#define ERROR_NAME "ZHEMM "
#else
#define ERROR_NAME "CHEMM "
#endif

static int (*hemm[])(long, long, long, FLOAT, FLOAT, FLOAT *, long, 
		FLOAT *, long, FLOAT *, long, FLOAT *) = {
#ifdef XDOUBLE
    xhemm_LU, xhemm_LL, xhemm_RU, xhemm_RL,
#elif defined(DOUBLE)
    zhemm_LU, zhemm_LL, zhemm_RU, zhemm_RL,
#else
    chemm_LU, chemm_LL, chemm_RU, chemm_RL,
#endif
  };

int NAME(char *SIDE, char *UPLO, 
         blasint *M, blasint *N, 
         FLOAT *ALPHA, FLOAT *a, blasint *LDA, 
         FLOAT *b, blasint *LDB, 
         FLOAT *BETA,  FLOAT *c, blasint *LDC){
  
  char side_arg  = *SIDE;
  char uplo_arg  = *UPLO;

  blasint m = *M;
  blasint n = *N;
  FLOAT alpha_r = *(ALPHA + 0);
  FLOAT alpha_i = *(ALPHA + 1);
  FLOAT beta_r  = *(BETA  + 0);
  FLOAT beta_i  = *(BETA  + 1);
  blasint lda = *LDA;
  blasint ldb = *LDB;
  blasint ldc = *LDC;
  FLOAT *buffer;

#ifdef SMP
  int mode;
#endif

  blasint info;
  int side;
  int uplo;
  int nrowa;

  side  = -1;
  uplo  = -1;

  TOUPPER(side_arg);
  TOUPPER(uplo_arg);

  if (side_arg  == 'L') side  = 0;
  if (side_arg  == 'R') side  = 1;

  if (uplo_arg  == 'U') uplo  = 0;
  if (uplo_arg  == 'L') uplo  = 1;
  
  nrowa = m;
  if (side & 1) nrowa = n;

  info = 0;

  if (ldc < MAX(1,m))     info = 12;
  if (ldb < MAX(1,m))     info =  9;
  if (lda < MAX(1,nrowa)) info =  7;
  if (n < 0)              info =  4;
  if (m < 0)              info =  3;
  if (uplo  < 0)          info =  2;
  if (side  < 0)          info =  1;

  if (info != 0) {
    BLASFUNC(xerbla)(ERROR_NAME, &info, sizeof(ERROR_NAME));
    return 0;
  }
  
  if (m == 0 || n == 0) return 0;
  
#ifdef SMP
  if (blas_cpu_number == 0) blas_get_cpu_number();
#ifdef SMP_SERVER
  if (blas_server_avail == 0) blas_thread_init();
#endif
#endif

#ifndef SMP
  if ((beta_r != ONE) || (beta_i != ZERO))
    GEMM_BETA(m, n, 0, beta_r, beta_i, NULL, 0, NULL, 0, c, ldc);
#else

#ifdef DOUBLE
  mode  =  BLAS_DOUBLE | BLAS_COMPLEX;
#else
  mode  =  BLAS_SINGLE | BLAS_COMPLEX; 
#endif  
  
  if ((beta_r != ONE) || (beta_i != ZERO))
    beta_thread(mode, m, n, beta_r, beta_i, c, ldc, (void *)GEMM_BETA);

#endif

  if ((alpha_r == ZERO) && (alpha_i == ZERO)) return 0;

  buffer = (FLOAT *)blas_memory_alloc(0);

#ifndef SMP

  (hemm[(side << 1) | uplo ])
    (m, n, 0, alpha_r, alpha_i, a, lda, b, ldb, c, ldc, buffer);
  
#else

  mode |= (side  << BLAS_RSIDE_SHIFT);

  symm_thread(mode, m, n, alpha_r, alpha_i,
	      a, lda, b, ldb, c, ldc,
	      (void *)hemm[(side << 1) | uplo ], buffer);
#endif

  blas_memory_free(buffer);

  return 0;
}

==> GotoBLAS-1.12.tar.gz: GotoBLAS/level3/hemm/zhemm.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

#ifdef XDOUBLE
#define ERROR_NAME "XHEMM "
#elif defined(DOUBLE)
#define ERROR_NAME "ZHEMM "
#else
#define ERROR_NAME "CHEMM "
#endif

static int (*hemm[])(long, long, long, FLOAT, FLOAT, FLOAT *, long, 
		FLOAT *, long, FLOAT *, long, FLOAT *) = {
#ifdef XDOUBLE
    xhemm_LU, xhemm_LL, xhemm_RU, xhemm_RL,
#elif defined(DOUBLE)
    zhemm_LU, zhemm_LL, zhemm_RU, zhemm_RL,
#else
    chemm_LU, chemm_LL, chemm_RU, chemm_RL,
#endif
  };

int NAME(char *SIDE, char *UPLO, 
         blasint *M, blasint *N, 
         FLOAT *ALPHA, FLOAT *a, blasint *LDA, 
         FLOAT *b, blasint *LDB, 
         FLOAT *BETA,  FLOAT *c, blasint *LDC){
  
  char side_arg  = *SIDE;
  char uplo_arg  = *UPLO;

  blasint m = *M;
  blasint n = *N;
  FLOAT alpha_r = *(ALPHA + 0);
  FLOAT alpha_i = *(ALPHA + 1);
  FLOAT beta_r  = *(BETA  + 0);
  FLOAT beta_i  = *(BETA  + 1);
  blasint lda = *LDA;
  blasint ldb = *LDB;
  blasint ldc = *LDC;
  FLOAT *buffer;

#ifdef SMP
  int mode;
#endif

  blasint info;
  int side;
  int uplo;
  int nrowa;

  side  = -1;
  uplo  = -1;

  TOUPPER(side_arg);
  TOUPPER(uplo_arg);

  if (side_arg  == 'L') side  = 0;
  if (side_arg  == 'R') side  = 1;

  if (uplo_arg  == 'U') uplo  = 0;
  if (uplo_arg  == 'L') uplo  = 1;
  
  nrowa = m;
  if (side & 1) nrowa = n;

  info = 0;

  if (ldc < MAX(1,m))     info = 12;
  if (ldb < MAX(1,m))     info =  9;
  if (lda < MAX(1,nrowa)) info =  7;
  if (n < 0)              info =  4;
  if (m < 0)              info =  3;
  if (uplo  < 0)          info =  2;
  if (side  < 0)          info =  1;

  if (info != 0) {
    BLASFUNC(xerbla)(ERROR_NAME, &info, sizeof(ERROR_NAME));
    return 0;
  }
  
  if (m == 0 || n == 0) return 0;
  
#ifdef SMP
  if (blas_cpu_number == 0) blas_get_cpu_number();
#ifdef SMP_SERVER
  if (blas_server_avail == 0) blas_thread_init();
#endif
#endif

#ifndef SMP
  if ((beta_r != ONE) || (beta_i != ZERO))
    GEMM_BETA(m, n, 0, beta_r, beta_i, NULL, 0, NULL, 0, c, ldc);
#else

#ifdef XDOUBLE
  mode  =  BLAS_XDOUBLE | BLAS_COMPLEX;
#elif defined(DOUBLE)
  mode  =  BLAS_DOUBLE  | BLAS_COMPLEX;
#else
  mode  =  BLAS_SINGLE  | BLAS_COMPLEX;
#endif  
  
  if ((beta_r != ONE) || (beta_i != ZERO))
    beta_thread(mode, m, n, beta_r, beta_i, c, ldc, (void *)GEMM_BETA);

#endif

  if ((alpha_r == ZERO) && (alpha_i == ZERO)) return 0;

  buffer = (FLOAT *)blas_memory_alloc(0);

#ifndef SMP

  (hemm[(side << 1) | uplo ])
    (m, n, 0, alpha_r, alpha_i, a, lda, b, ldb, c, ldc, buffer);
  
#else

  mode |= (side  << BLAS_RSIDE_SHIFT);

  symm_thread(mode, m, n, alpha_r, alpha_i,
	      a, lda, b, ldb, c, ldc,
	      (void *)hemm[(side << 1) | uplo ], buffer);
#endif

  blas_memory_free(buffer);

  return 0;
}
